#!/bin/bash
# 
#----------------------------------------------------------------------------
# /var/install/config.d/net-snmp-update.sh
#
# Creation:     05.07.2005  OJae
# Last Update:  31.08.2020  OJae
#
# Copyright (c) 2005-2020 Olaf Jaehrling <eisfair@ojaehrling.de>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License
#----------------------------------------------------------------------------

package='net-snmp'
package_name='net-snmp'
configfile=/etc/config.d/$package
source_conf=/etc/config.d/$package
# include eislib
. /var/install/include/eislib
. /var/install/include/configlib

#exec 2>/tmp/snmp_up-trace-$$.log
#set -x

### ---------------------------------------------------------------------------
### Set the default values for configuration
### ---------------------------------------------------------------------------

#------------------------------------------------------------------------------
#                               net-snmp setup Menu
#------------------------------------------------------------------------------

START_NET_SNMP='yes'                    # start net_smnp at boot the system
NET_SNMP_LOCATION_NAME='Server room'    # the name of yout Serverroom, Default: Sysadmin
NET_SNMP_ADMIN_NAME='Sysadmin'          # the Sysadminname, Default: Sysadmin
NET_SNMP_ADMIN_MAIL='root@localhost'    # mail from sysadmin
NET_SNMP_LOG='yes'
NET_SNMP_OWN_LOGFILE='no'
NET_SNMP_LOGFILE='/var/log/snmp'
NET_SNMP_OPTIONS=''
NET_SNMP_USE_V3='no'
NET_SNMP_RO_ALLOWED_N='1'                     # number of community
NET_SNMP_RW_ALLOWED_N='0'                     # number of community
# only for V3
NET_SNMP_V3_RO_N='0'
NET_SNMP_V3_RO_1_USERNAME='eis'
NET_SNMP_V3_RO_1_PASSWORD='pass'
NET_SNMP_V3_RO_1_AUTH='yes'
NET_SNMP_V3_RO_1_AUTH_PROTOCOL='SHA256'
NET_SNMP_V3_RO_1_PRIV='yes'
NET_SNMP_V3_RO_1_PRIV_PROTOCOL='DES'
NET_SNMP_V3_RO_1_PRIV_PASS='pass'
#NET_SNMP_V3_RO_1_IP=''
NET_SNMP_V3_RW_N='0'
NET_SNMP_V3_RW_1_USERNAME='eis'
NET_SNMP_V3_RW_1_PASSWORD='pass'
NET_SNMP_V3_RW_1_AUTH='yes'
NET_SNMP_V3_RW_1_AUTH_PROTOCOL='SHA256'
NET_SNMP_V3_RW_1_PRIV='yes'
NET_SNMP_V3_RW_1_PRIV_PROTOCOL='DES'
NET_SNMP_V3_RW_1_PRIV_PASS='pass'
#NET_SNMP_V3_RW_1_IP=''

# only for V1,2c
NET_SNMP_RO_1_COMMUNITY_NAME='public'         # name of community
NET_SNMP_RO_1_ALLOWED_IP='127.0.0.1'          # IP address of community
NET_SNMP_RO_1_USERNAME='eis'
NET_SNMP_RO_1_PASSWORD='pass'
NET_SNMP_RW_1_COMMUNITY_NAME='public'         # name of community
NET_SNMP_RW_1_ALLOWED_IP='127.0.0.1'          # IP address of community
NET_SNMP_RW_1_USERNAME='eis'
NET_SNMP_RW_1_PASSWORD='pass'


### ---------------------------------------------------------------------------
### read old configuration and rename old variables
### ---------------------------------------------------------------------------
read_old_variables()
{
    # read old values
    if [ -f /etc/config.d/${package_name} ]
    then
        . /etc/config.d/${package_name}
    fi
}

delete_old_net_snmp ()
{
    if [ -f /etc/config.d/net_snmp ]
    then
        . /etc/config.d/net_snmp
        mecho -std "remove old net_snmp_package"
        /var/install/bin/del-package net_snmp >/dev/null 2>&1
        rm -rf /etc/backup.d/net_snmp*
        rm -f /etc/check.d/net_snmp
        rm -f /etc/check.d/net_snmp.exp
        rm -f /etc/config.d/net_snmp
        rm -f /etc/default.d/net_snmp
        rm -f /etc/filelist.d/net_snmp-files.txt
        rm -f /etc/init.d/net_snmp
        rm -f /etc/rc2.d/K25net_snmp
        rm -f /etc/rc2.d/S75net_snmp
        rm -rf /usr/share/doc/net_snmp
        rm -f /var/install/config.d/net_snmp-update.sh
        rm -f /var/install/config.d/net_snmp.sh
        rm -f /var/install/deinstall/net_snmp
        rm -f /var/install/help/net_snmp
        rm -f /var/install/packages/net_snmp
 
        /var/install/bin/del-menu              \
        setup.services.menu                \
        setup.services.net_snmp.menu       \
        net_snmp
        rm -f /var/install/menu/setup.services.net_snmp.menu
        rm -f /var/install/bin/setup.services.net_snmp
  
        if [ -f /var/install/deinstall/net_snmp_mibs ]
        then 
           rm -f /var/install/deinstall/net_snmp_mibs
           rm -f /etc/filelist.d/net_snmp_mibs-files.txt
           rm -rf /usr/share/doc/net_snmp_mibs
           rm -f /var/install/help/net_snmp_mibs
           rm -f /var/install/packages/net_snmp_mibs
        fi
        if [ -f /var/install/deinstall/net_snmp_lib ]
        then
           rm -f /var/install/deinstall/net_snmp_lib
           rm -f /etc/filelist.d/net_snmp_lib-files.txt
           rm -rf /usr/share/doc/net_snmp_mibs
           rm -f /var/install/packages/net_snmp_mibs
        fi
    fi
    
}

#------------------------------------------------------------------------
# modify variables
#------------------------------------------------------------------------

modify_variables ()
{

    modified=0
    mecho -info "modifying parameter(s) ..."
}

#------------------------------------------------------------------------
# create new configuration
#------------------------------------------------------------------------
### ---------------------------------------------------------------------------
### Write config and default files
### ---------------------------------------------------------------------------
make_config_file()
{
    internal_conf_file=${1}
    (
    #--------------------------------------------------------------------------------
    printgpl -conf ${package} `date --rfc-3339=date` "OJae" "Olaf Jaehrling <eisfair[at]ojaehrling[dot]de>" 
    #--------------------------------------------------------------------------------
    #--------------------------------------------------------------------------------
    printgroup "               net-snmp setup Menu                    "
    #--------------------------------------------------------------------------------

    printvar "START_NET_SNMP"          "start net_smnp at boot the system"
    printvar "NET_SNMP_LOCATION_NAME"  "the name of yout Serverroom, Default: Sysadmin"
    printvar "NET_SNMP_ADMIN_NAME"     "the Sysadminname, Default: Sysadmin"
    printvar "NET_SNMP_ADMIN_MAIL"     "mail from sysadmin"
    printvar "NET_SNMP_LOG"            "log snmprequests in a logfile? yes/no"
    printvar "NET_SNMP_OWN_LOGFILE"         "log in the syslog?"
    printvar "NET_SNMP_LOGFILE"        "specify the logfile"
    printvar "NET_SNMP_OPTIONS"        "additionals Options"
    printvar "NET_SNMP_USE_V3"         "snmp use version 3?"

    printvar "NET_SNMP_RO_ALLOWED_N"         "number of community"
    idx=1
    while [ "$idx" -le "$NET_SNMP_RO_ALLOWED_N" ]
    do
       printvar 'NET_SNMP_RO_'$idx'_COMMUNITY_NAME'    "name of community"
       printvar 'NET_SNMP_RO_'$idx'_ALLOWED_IP'        "IP address of community"
    idx=`/usr/bin/expr $idx + 1`
    done

    printvar "NET_SNMP_V3_RO_N"         "number of readonly user"
    idx=1
    while [ "$idx" -le "$NET_SNMP_V3_RO_N" ]
    do
       printvar 'NET_SNMP_V3_RO_'$idx'_USERNAME'       ""
       printvar 'NET_SNMP_V3_RO_'$idx'_PASSWORD'       ""
       printvar 'NET_SNMP_V3_RO_'$idx'_AUTH'           ""
       printvar 'NET_SNMP_V3_RO_'$idx'_AUTH_PROTOCOL'  ""
       printvar 'NET_SNMP_V3_RO_'$idx'_PRIV'           ""
       printvar 'NET_SNMP_V3_RO_'$idx'_PRIV_PROTOCOL'  ""
       printvar 'NET_SNMP_V3_RO_'$idx'_PRIV_PASS'      ""
#       printvar 'NET_SNMP_V3_RO_'$idx'_IP'    ""
    idx=`/usr/bin/expr $idx + 1`
    done

    printgroup " "

    printvar "NET_SNMP_RW_ALLOWED_N"         "number of community"
    idx1=1
    while [ "$idx1" -le "$NET_SNMP_RW_ALLOWED_N" ]
    do
       printvar 'NET_SNMP_RW_'$idx1'_COMMUNITY_NAME'       "name of community"
       printvar 'NET_SNMP_RW_'$idx1'_ALLOWED_IP'           "IP address of community"
    idx1=`/usr/bin/expr $idx1 + 1`
    done

    printvar "NET_SNMP_V3_RW_N"         "number of readwrite user"
    idx1=1
    while [ "$idx1" -le "$NET_SNMP_V3_RW_N" ]
    do
       printvar 'NET_SNMP_V3_RW_'$idx1'_USERNAME'       ""
       printvar 'NET_SNMP_V3_RW_'$idx1'_PASSWORD'       ""
       printvar 'NET_SNMP_V3_RW"_'$idx1'_AUTH'           ""
       printvar 'NET_SNMP_V3_RW_'$idx1'_AUTH_PROTOCOL'  ""
       printvar 'NET_SNMP_V3_RW_'$idx1'_PRIV'           ""
       printvar 'NET_SNMP_V3_RW_'$idx1'_PRIV_PROTOCOL'  ""
       printvar 'NET_SNMP_V3_RW_'$idx1'_PRIV_PASS'      ""
#       printvar 'NET_SNMP_V3_RW_'$idx1'_IP'    ""
    idx1=`/usr/bin/expr $idx1 + 1`
    done
    #--------------------------------------------------------------------------------
    printend
    #--------------------------------------------------------------------------------

    ) > ${internal_conf_file}
    # Set rights
    chmod 0600 ${internal_conf_file}
    chown root ${internal_conf_file}
}

### -------------------------------------------------------------------------
### Create the check.d file
### -------------------------------------------------------------------------

make_check_file()
{
#--------------------------------------------------------------------------------
printgpl -check ${package_name} `date --rfc-3339=date` "OJae" "Olaf Jaehrling <eisfair[at]ojaehrling[dot]de>" >/etc/check.d/${package_name}

cat >>/etc/check.d/${package_name} <<EOF
# Variable                     OPT_VARIABLE            VARIABLE_N               VALUE
START_NET_SNMP                  -                      -                        YESNO
NET_SNMP_LOCATION_NAME         START_NET_SNMP          -                        NOTEMPTY
NET_SNMP_ADMIN_NAME            START_NET_SNMP          -                        NOTEMPTY
NET_SNMP_ADMIN_MAIL            START_NET_SNMP          -                        MAILADDR
NET_SNMP_LOG                   START_NET_SNMP          -                        YESNO
NET_SNMP_OWN_LOGFILE           NET_SNMP_LOG            -                        YESNO
NET_SNMP_LOGFILE               NET_SNMP_OWN_LOGFILE    -                        ABS_PATH
NET_SNMP_USE_V3                START_NET_SNMP          -                        YESNO
NET_SNMP_OPTIONS               START_NET_SNMP          -                        NONE
NET_SNMP_RO_ALLOWED_N          START_NET_SNMP          -                        NUMERIC
NET_SNMP_RO_%_COMMUNITY_NAME   START_NET_SNMP        NET_SNMP_RO_ALLOWED_N      NOTEMPTY
NET_SNMP_RO_%_ALLOWED_IP       START_NET_SNMP        NET_SNMP_RO_ALLOWED_N      SNMPIPADDR
NET_SNMP_V3_RO_N               NET_SNMP_USE_V3         -                        NUMERIC
NET_SNMP_V3_RO_%_USERNAME      NET_SNMP_USE_V3       NET_SNMP_V3_RO_N   NOTEMPTY
NET_SNMP_V3_RO_%_PASSWORD      NET_SNMP_USE_V3       NET_SNMP_V3_RO_N   SNMPPASS
NET_SNMP_V3_RO_%_AUTH          NET_SNMP_USE_V3       NET_SNMP_V3_RO_N   YESNO
NET_SNMP_V3_RO_%_AUTH_PROTOCOL NET_SNMP_V3_RO_%_AUTH NET_SNMP_V3_RO_N   SNMPAPRO
NET_SNMP_V3_RO_%_PRIV          NET_SNMP_USE_V3       NET_SNMP_V3_RO_N   YESNO
NET_SNMP_V3_RO_%_PRIV_PROTOCOL NET_SNMP_V3_RO_%_PRIV NET_SNMP_V3_RO_N   SNMPPPRO
NET_SNMP_V3_RO_%_PRIV_PASS     NET_SNMP_V3_RO_%_PRIV NET_SNMP_V3_RO_N   SNMPPASS
#NET_SNMP_V3_RO_%_ALLOWED_IP    NET_SNMP_USE_V3       NET_SNMP_V3_RO_N   SNMPIPADDR
NET_SNMP_RW_ALLOWED_N          START_NET_SNMP          -                        NUMERIC
NET_SNMP_RW_%_COMMUNITY_NAME   START_NET_SNMP        NET_SNMP_RW_ALLOWED_N      NOTEMPTY
NET_SNMP_RW_%_ALLOWED_IP       START_NET_SNMP        NET_SNMP_RW_ALLOWED_N      SNMPIPADDR
NET_SNMP_V3_RW_N       NET_SNMP_USE_V3         -                        NUMERIC
NET_SNMP_V3_RW_%_USERNAME      NET_SNMP_USE_V3       NET_SNMP_V3_RW_N   NOTEMPTY
NET_SNMP_V3_RW_%_PASSWORD      NET_SNMP_USE_V3       NET_SNMP_V3_RW_N   SNMPPASS
NET_SNMP_V3_RW_%_AUTH          NET_SNMP_USE_V3       NET_SNMP_V3_RW_N   YESNO
NET_SNMP_V3_RW_%_AUTH_PROTOCOL NET_SNMP_V3_RW_%_AUTH NET_SNMP_V3_RW_N   SNMPAPRO
NET_SNMP_V3_RW_%_PRIV          NET_SNMP_USE_V3       NET_SNMP_V3_RW_N   YESNO
NET_SNMP_V3_RW_%_PRIV_PROTOCOL NET_SNMP_V3_RW_%_PRIV NET_SNMP_V3_RW_N   SNMPPPRO
NET_SNMP_V3_RW_%_PRIV_PASS     NET_SNMP_V3_RW_%_PRIV NET_SNMP_V3_RW_N   SNMPPASS
#NET_SNMP_V3_RW_%_ALLOWED_IP    NET_SNMP_USE_V3       NET_SNMP_V3_RW_N   SNMPIPADDR

EOF

    # Set rights for check.d file
    chmod 0600 /etc/check.d/${package_name}
    chown root /etc/check.d/${package_name}
}
## -------------------------------------------------------------------------
### Create the exp file
### -------------------------------------------------------------------------
make_exp_file()
{
#--------------------------------------------------------------------------------
printgpl -check_exp ${package_name} `date --rfc-3339=date` "OJae" "Olaf Jaehrling <eisfair[at]ojaehrling[dot]de>" >/etc/check.d/${package_name}.exp
#--------------------------------------------------------------------------------
cat >>/etc/check.d/${package_name}.exp <<EOF
SNMPVER = '()|1|2c|3' : 'only blan, 1,2c or 3 allowed'
SNMPAPRO = 'MD5|SHA1|SHA512|SHA224|SHA384|SHA256' : 'only MD5 SHA1 SHA512 SHA224 SHA384 SHA256 are allowed'
SNMPPPRO = '()|DES|AES|AES128' : 'only blank, DES AES or AES128 are allowed'
SNMPIPADDR = '(RE:EIPADDR)|(RE:EIPADDR6)|(RE:EIPADDR6COMPAT)' : 'only blanc, ipv4 or ipv6 address are allowed'
SNMPPASS = '()|(^.{8,})' : 'password must be set blanc or more then 7 digit'


EOF
    # Set rights for exp file
    chmod 0600 /etc/check.d/${package_name}.exp
    chown root /etc/check.d/${package_name}.exp
}
### -------------------------------------------------------------------------
### Create the ext file
### -------------------------------------------------------------------------

make_ext_file()
{
#--------------------------------------------------------------------------------
printgpl -check_ext ${package_name} `date --rfc-3339=date` "OJae" "Olaf Jaehrling <eisfair[at]ojaehrling[dot]de>" >/etc/check.d/${package_name}.ext
#--------------------------------------------------------------------------------
cat >>/etc/check.d/${package_name}.ext <<EOF
# read ro users
set ro_userlist=""
foreach idx in net_snmp_v3_ro_n
do
    set username = net_snmp_v3_ro_%_username[idx]

    if (ro_userlist == "")
    then
        set ro_userlist="\$username"
    else
        set ro_userlist="\$ro_userlist:\$username"
    fi

    set ro_max = idx
done

# read rw users
set rw_userlist=""
foreach idx in net_snmp_v3_rw_n
do
    set username = net_snmp_v3_rw_%_username[idx]

    if (rw_userlist == "")
    then
        set rw_userlist="\$username"
    else
        set rw_userlist="\$rw_userlist:\$username"
    fi

    set rw_max = idx
done

# check for double users
split(ro_userlist, ro_userlist_%, ':')
split(rw_userlist, rw_userlist_%, ':')

set break = 0
foreach idx in net_snmp_v3_ro_n
do
    if (break == 0)
    then
        set user_x = net_snmp_v3_ro_%_username[idx]

        set jdx = 1
        foreach user_y in ro_userlist_%
        do
            if (break == 0)
            then
                if (jdx != idx)
                then
                    if (user_x == user_y)
                    then
                        set break = 1
                        warning "Doppelter RO-Eintrag '\$idx-\$user_x'=='\$jdx-\$user_y'"
                    fi
                fi
                set jdx = jdx + 1
            fi
        done
    fi
done

if (break == 0)
then
    foreach idx in net_snmp_v3_rw_n
    do
        if (break == 0)
        then
            set user_x = net_snmp_v3_rw_%_username[idx]

            set jdx = 1
            foreach user_y in rw_userlist_%
            do
                if (break == 0)
                then
                    if (jdx != idx)
                    then
                        if (user_x == user_y)
                        then
                            set break = 1
                            warning "Doppelter RW-Eintrag '\$idx-\$user_x'=='\$jdx-\$user_y'"
                        fi
                    fi
                    set jdx = jdx + 1
                fi
            done
        fi
    done
fi

if (break == 0)
then
    foreach idx in net_snmp_v3_ro_n
    do
        if (break == 0)
        then
            set user_x = net_snmp_v3_ro_%_username[idx]

            set jdx = 1
            foreach user_y in rw_userlist_%
            do
                if (break == 0)
                then
                    if (jdx != idx)
                    then
                        if (user_x == user_y)
                        then
                            set break = 1
                            warning "Doppelter RO/RW-Eintrag '\$idx-\$user_x'=='\$jdx-\$user_y'"
                        fi
                    fi
                    set jdx = jdx + 1
                fi
            done
        fi
    done
fi


EOF

    # Set rights for ext file
    chmod 0600 /etc/check.d/${package_name}.ext
    chown root /etc/check.d/${package_name}.ext
}



### ---------------------------------------------------------------------------
### Main
### ---------------------------------------------------------------------------

# delete old net_snmp paket
#delete_old_net_snmp

# update from old version
read_old_variables

# write default config file
make_config_file /etc/default.d/${package_name}

# write new config file
make_config_file /etc/config.d/${package_name}

# write check.d file
make_check_file

# write *.exp file
make_exp_file

# write *.ext file
make_ext_file

#mecho -info "modifying parameter(s) ..."
#        mecho -warn "... read documentation for modified and/or added parameter(s)!"
#        mecho -warn " please check the configfile and remove obsolete space character"
#
#anykey

exit 0

### ---------------------------------------------------------------------------
### End
###----------------------------------------------------------------------------



